@@ -144,6 +144,11 @@ struct expression : public visitable
   const token* tok;
   expression ();
   virtual ~expression ();
+  // NB: this pretty-printer function must generate such text that
+  // allows distinguishing operationally-different instances.  During
+  // stap -p2/-p3, function bodies with identical pretty-printed
+  // bodies may be duplicate-eliminated.  So print any relevant member
+  // variables somehow.
   virtual void print (std::ostream& o) const = 0;
   virtual void visit (visitor* u) = 0;
   virtual bool is_symbol(symbol *& sym_out);
@@ -247,12 +252,17 @@ struct regex_query: public expression
   void print (std::ostream& o) const;
 };
 
+struct compound_expression: public binary_expression
+{
+  compound_expression() { op = ","; }
+  void visit (visitor* u);
+};
+
 struct comparison: public binary_expression
 {
   void visit (visitor* u);
 };
 
-
 struct concatenation: public binary_expression
 {
   void visit (visitor* u);
@@ -305,6 +315,33 @@ struct symbol: public indexable
   bool is_symbol(symbol *& sym_out);
 };
 
+struct target_register: public expression
+{
+  unsigned regno;
+  bool userspace_p;
+  void print (std::ostream& o) const;
+  void visit (visitor* u);
+};
+
+struct target_deref: public expression
+{
+  expression* addr;
+  unsigned size;
+  bool signed_p;
+  bool userspace_p;
+  void print (std::ostream& o) const;
+  void visit (visitor* u);
+};
+
+struct target_bitfield: public expression
+{
+  expression* base;
+  unsigned offset;
+  unsigned size;
+  bool signed_p;
+  void print (std::ostream& o) const;
+  void visit (visitor* u);
+};
 
 struct target_symbol: public expression
 {
@@ -904,11 +941,15 @@ struct visitor
   virtual void visit_logical_and_expr (logical_and_expr* e) = 0;
   virtual void visit_array_in (array_in* e) = 0;
   virtual void visit_regex_query (regex_query* e) = 0;
+  virtual void visit_compound_expression (compound_expression * e) = 0;
   virtual void visit_comparison (comparison* e) = 0;
   virtual void visit_concatenation (concatenation* e) = 0;
   virtual void visit_ternary_expression (ternary_expression* e) = 0;
   virtual void visit_assignment (assignment* e) = 0;
   virtual void visit_symbol (symbol* e) = 0;
+  virtual void visit_target_register (target_register* e) = 0;
+  virtual void visit_target_deref (target_deref* e) = 0;
+  virtual void visit_target_bitfield (target_bitfield* e) = 0;
   virtual void visit_target_symbol (target_symbol* e) = 0;
   virtual void visit_arrayindex (arrayindex* e) = 0;
   virtual void visit_functioncall (functioncall* e) = 0;
@@ -953,11 +994,15 @@ struct nop_visitor: public visitor
   virtual void visit_logical_and_expr (logical_and_expr*) {};
   virtual void visit_array_in (array_in*) {};
   virtual void visit_regex_query (regex_query*) {};
+  virtual void visit_compound_expression (compound_expression *) {};
   virtual void visit_comparison (comparison*) {};
   virtual void visit_concatenation (concatenation*) {};
   virtual void visit_ternary_expression (ternary_expression*) {};
   virtual void visit_assignment (assignment*) {};
   virtual void visit_symbol (symbol*) {};
+  virtual void visit_target_register (target_register*) {};
+  virtual void visit_target_deref (target_deref*) {};
+  virtual void visit_target_bitfield (target_bitfield*) {};
   virtual void visit_target_symbol (target_symbol*) {};
   virtual void visit_arrayindex (arrayindex*) {};
   virtual void visit_functioncall (functioncall*) {};
@@ -1002,11 +1047,15 @@ struct traversing_visitor: public visitor
   void visit_logical_and_expr (logical_and_expr* e);
   void visit_array_in (array_in* e);
   void visit_regex_query (regex_query* e);
+  void visit_compound_expression(compound_expression* e);
   void visit_comparison (comparison* e);
   void visit_concatenation (concatenation* e);
   void visit_ternary_expression (ternary_expression* e);
   void visit_assignment (assignment* e);
   void visit_symbol (symbol* e);
+  void visit_target_register (target_register* e);
+  void visit_target_deref (target_deref* e);
+  void visit_target_bitfield (target_bitfield* e);
   void visit_target_symbol (target_symbol* e);
   void visit_arrayindex (arrayindex* e);
   void visit_functioncall (functioncall* e);
@@ -1038,11 +1087,15 @@ struct expression_visitor: public traversing_visitor
   void visit_logical_and_expr (logical_and_expr* e);
   void visit_array_in (array_in* e);
   void visit_regex_query (regex_query* e);
+  void visit_compound_expression (compound_expression* e);
   void visit_comparison (comparison* e);
   void visit_concatenation (concatenation* e);
   void visit_ternary_expression (ternary_expression* e);
   void visit_assignment (assignment* e);
   void visit_symbol (symbol* e);
+  void visit_target_register (target_register* e);
+  void visit_target_deref (target_deref* e);
+  void visit_target_bitfield (target_bitfield* e);
   void visit_target_symbol (target_symbol* e);
   void visit_arrayindex (arrayindex* e);
   void visit_functioncall (functioncall* e);
@@ -1104,6 +1157,8 @@ struct varuse_collecting_visitor: public functioncall_traversing_visitor
   void visit_assignment (assignment *e);
   void visit_ternary_expression (ternary_expression* e);
   void visit_arrayindex (arrayindex *e);
+  void visit_target_register (target_register* e);
+  void visit_target_deref (target_deref* e);
   void visit_target_symbol (target_symbol *e);
   void visit_symbol (symbol *e);
   void visit_pre_crement (pre_crement *e);
@@ -1155,11 +1210,15 @@ struct throwing_visitor: public visitor
   void visit_logical_and_expr (logical_and_expr* e);
   void visit_array_in (array_in* e);
   void visit_regex_query (regex_query* e);
+  void visit_compound_expression (compound_expression* e);
   void visit_comparison (comparison* e);
   void visit_concatenation (concatenation* e);
   void visit_ternary_expression (ternary_expression* e);
   void visit_assignment (assignment* e);
   void visit_symbol (symbol* e);
+  void visit_target_register (target_register* e);
+  void visit_target_deref (target_deref* e);
+  void visit_target_bitfield (target_bitfield* e);
   void visit_target_symbol (target_symbol* e);
   void visit_arrayindex (arrayindex* e);
   void visit_functioncall (functioncall* e);
@@ -1220,22 +1279,21 @@ struct update_visitor: public visitor
         T* new_src = require(src, clearok);
         if (old_src != new_src)
           {
-            #if 0 /* XXX: this should be parametrized from session.verbose, and
-                     could take place of all of those 'elided ...' messages
-                     in elaborate.cxx optimization visitors. */
-            std::cout << "replaced ";
-            old_src->print(std::cout);
-            std::cout << " with ";
-            new_src->print(std::cout);
-            std::cout << std::endl;
-            #endif
+	    if (this->verbose > 3)
+	      {
+		std::clog << _("replaced ");
+		old_src->print(std::clog);
+		std::clog << _(" with ");
+		new_src->print(std::clog);
+		std::clog << std::endl;
+	      }
             relaxed_p = false;
           }
         src = new_src;
       }
   }
 
-  update_visitor(): aborted_p(false), relaxed_p(true) {}
+  update_visitor(unsigned v = 0): verbose(v), aborted_p(false), relaxed_p(true) {}
   virtual ~update_visitor() { assert(values.empty()); }
 
   // Permit reuse of the visitor object. 
@@ -1266,11 +1324,15 @@ struct update_visitor: public visitor
   virtual void visit_logical_and_expr (logical_and_expr* e);
   virtual void visit_array_in (array_in* e);
   virtual void visit_regex_query (regex_query* e);
+  virtual void visit_compound_expression (compound_expression* e);
   virtual void visit_comparison (comparison* e);
   virtual void visit_concatenation (concatenation* e);
   virtual void visit_ternary_expression (ternary_expression* e);
   virtual void visit_assignment (assignment* e);
   virtual void visit_symbol (symbol* e);
+  virtual void visit_target_register (target_register* e);
+  virtual void visit_target_deref (target_deref* e);
+  virtual void visit_target_bitfield (target_bitfield* e);
   virtual void visit_target_symbol (target_symbol* e);
   virtual void visit_arrayindex (arrayindex* e);
   virtual void visit_functioncall (functioncall* e);
@@ -1287,6 +1349,7 @@ struct update_visitor: public visitor
 private:
   std::stack<visitable *> values;
 protected:
+  unsigned verbose;
   bool aborted_p;
   bool relaxed_p;
 };
@@ -1329,11 +1392,15 @@ struct deep_copy_visitor: public update_visitor
   virtual void visit_logical_and_expr (logical_and_expr* e);
   virtual void visit_array_in (array_in* e);
   virtual void visit_regex_query (regex_query* e);
+  virtual void visit_compound_expression(compound_expression* e);
   virtual void visit_comparison (comparison* e);
   virtual void visit_concatenation (concatenation* e);
   virtual void visit_ternary_expression (ternary_expression* e);
   virtual void visit_assignment (assignment* e);
   virtual void visit_symbol (symbol* e);
+  virtual void visit_target_register (target_register* e);
+  virtual void visit_target_deref (target_deref* e);
+  virtual void visit_target_bitfield (target_bitfield* e);
   virtual void visit_target_symbol (target_symbol* e);
   virtual void visit_arrayindex (arrayindex* e);
   virtual void visit_functioncall (functioncall* e);
